# -*- coding: utf-8 -*-
# vim:fenc=utf-8
#
# Copyright © 2015-2020 Óscar García Amor <ogarcia@connectical.com>
#
# Distributed under terms of the GNU GPLv3 license.

NAME = 'trellowarrior'
VERSION = '1.2.0'
AUTHOR_NAME = 'Óscar García Amor'
AUTHOR_EMAIL = 'ogarcia@connectical.com'
DESCRIPTION = 'Tool to sync Taskwarrior projects with Trello boards'
KEYWORDS = 'board easy project sync task taskwarrior tool trello'
URL = 'https://github.com/ogarcia/trellowarrior'
LICENSE = 'GPLv3'
